from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional

from rubicon_ml.domain.utils import TrainingMetadata, uuid


@dataclass
class Project:
    name: str

    id: str = field(default_factory=uuid.uuid4)
    description: Optional[str] = None
    github_url: Optional[str] = None
    training_metadata: Optional[TrainingMetadata] = None
    created_at: datetime = field(default_factory=datetime.utcnow)



from typing import List


class TagMixin:
    """Adds tagging support to a domain model."""

    def add_tags(self, tags: List[str]):
        """
        Add new tags to this model.

        Parameters
        ----------
        tags : List[str]
            A list of string tags to add to the domain model.
        """
        self.tags = list(set(self.tags).union(set(tags)))

    def remove_tags(self, tags: List[str]):
        """
        Remove tags from this model.

        Parameters
        ----------
        tags : List[str]
            A list of string tags to remove from this domain model.
        """
        self.tags = list(set(self.tags).difference(set(tags)))


class CommentMixin:
    """Adds comment support to a domain model."""

    def add_comments(self, comments: List[str]):
        """
        Add new comments to this model.

        Parameters
        ----------
        comments : List[str]
            A list of string comments to add to the domain model.
        """
        self.comments.extend(comments)

    def remove_comments(self, comments: List[str]):
        """
        Remove comments from this model.

        Parameters
        ----------
        comments : List[str]
            A list of string comments to remove from this domain model.
        """
        self.comments = list(set(self.comments).difference(set(comments)))



from dataclasses import dataclass, field
from datetime import datetime
from typing import List, Optional

from rubicon_ml.domain.mixin import CommentMixin, TagMixin
from rubicon_ml.domain.utils import uuid

DIRECTIONALITY_VALUES = ["score", "loss"]


@dataclass
class Metric(TagMixin, CommentMixin):
    name: str
    value: float

    id: str = field(default_factory=uuid.uuid4)
    description: Optional[str] = None
    directionality: str = "score"
    created_at: datetime = field(default_factory=datetime.utcnow)
    tags: List[str] = field(default_factory=list)
    comments: List[str] = field(default_factory=list)

    def __post_init__(self):
        if self.directionality not in DIRECTIONALITY_VALUES:
            raise ValueError(f"metric directionality must be one of {DIRECTIONALITY_VALUES}")
